package com.zulutrade.sample.binaryoptions.trading.client;

import java.util.List;

import com.zulutrade.sample.binaryoptions.trading.client.configuration.ZuluTradeAuthenticationInfo;
import com.zulutrade.sample.binaryoptions.trading.client.configuration.ZuluTradingServerConfig;
import com.zulutrade.sample.binaryoptions.trading.client.trading.BinaryOptionsRestTradingService;
import com.zulutrade.sample.binaryoptions.trading.client.trading.BinaryOptionsTradingService;
import com.zulutrade.sample.binaryoptions.trading.client.trading.TradingClientException;
import com.zulutrade.sample.binaryoptions.trading.client.trading.dto.Position;
import com.zulutrade.sample.binaryoptions.trading.client.trading.dto.RegularOption;
import com.zulutrade.sample.binaryoptions.trading.client.trading.dto.TraderStatistics;
import com.zulutrade.sample.binaryoptions.trading.client.trading.dto.rest.request.OpenRegularPositionRequest;
import com.zulutrade.sample.binaryoptions.trading.client.trading.dto.rest.request.OpenSixtySecondsPositionRequest;
import com.zulutrade.sample.binaryoptions.trading.client.trading.dto.rest.request.RegularOptionsRequest;
import com.zulutrade.sample.binaryoptions.trading.client.utils.DateUtil;

/**
 * Demo application showing how to use ZuluTrading API for the Binary Options
 *
 * @author ZuluTrade
 * @version 1.0
 * @since 2015-06-15
 */
public class ZuluTradingBinaryOptionsSampleApp {

	public static void main(String[] args) {

		System.out.println("App starting");
	
		ZuluTradingServerConfig config = new ZuluTradingServerConfig("http://tradingserverbinary.zulutrade.com/");
		ZuluTradeAuthenticationInfo authInfo = new ZuluTradeAuthenticationInfo("yourUsername", "yourPassword");
		BinaryOptionsTradingService tradingService = new BinaryOptionsRestTradingService(config, authInfo);
		
		/*1. get the assets we can trade*/
		try {
			System.out.println("1. Requesting available assets");
			List<String> assets = tradingService.requestAssets();
			System.out.println("\tAssets:"+assets); 
		} catch (TradingClientException e) {
			System.out.println("Exception on requestAssets, error:"+e.getTradingError());
		}
		
		/*2. get the balance*/
		try {
			System.out.println("2. Requesting account balance");
			double balance = tradingService.requestBalance();
			System.out.println("\tBalance:"+balance); 
		} catch (TradingClientException e) {
			System.out.println("Exception on requestBalance, error:"+e.getTradingError());
		}
		
		/*3. get the open positions*/
		try {
			System.out.println("3. Requesting open positions");
			List<Position> openPositions = tradingService.requestOpenPositions();
			for(Position position:openPositions){
				System.out.println("\t"+position);
			}
		} catch (TradingClientException e) {
			System.out.println("Exception on requestOpenPositions, error:"+e.getTradingError());
		}
		
		/*4. get the closed positions*/
		try {
			System.out.println("4. Requesting closed positions");
			List<Position> closedPositions = tradingService.requestClosedPositions();
			for(Position position:closedPositions){
				System.out.println("\t"+position);
			}
		} catch (TradingClientException e) {
			System.out.println("Exception on requestClosedPositions, error:"+e.getTradingError());
		}
		
		/*5. get the closed positions by dates*/
		try {
			String startDate = DateUtil.getStartDate();
			String endDate = DateUtil.getEndDate();
			System.out.println("5. Requesting closed positions from: "+startDate+" until : "+endDate);
			List<Position> closedPositions = tradingService.requestClosedPositionsByDate(startDate, endDate);
			for(Position position:closedPositions){
				System.out.println("\t"+position);
			}
		} catch (TradingClientException e) {
			System.out.println("Exception on requestClosedPositions, error:"+e.getTradingError());
		}
		
		/*6. get the statistics for our trader account */
		try {
			System.out.println("6. Requesting our accounts's statistics");
			TraderStatistics stats = tradingService.requestTraderStatistics();
			System.out.println("\tTraderStatistics:"+stats); 
		} catch (TradingClientException e) {
			System.out.println("Exception on requestTraderStatistics, error:"+e.getTradingError());
		}
		
		/*7. get the top 5 traders*/
		List<TraderStatistics> topTraders = null; 
		try { 
			System.out.println("7. Requesting Zulutrade's top 5 traders");
			topTraders = tradingService.requestTopTrades(5); 
			for(TraderStatistics trader: topTraders){
				System.out.println("\t"+trader);
			}
		}
		catch(TradingClientException e){
			System.out.println("Exception on requestTopTrades, error:"+e.getTradingError());
		}
		
		/*8. open a 60 seconds position */
		try {
			String positionType = "put"; 
			double amount = 25;
			String asset = "EUR/USD"; 
			int seconds = 60;
			
			System.out.println("8. Sending 60seconds position opening request");
			OpenSixtySecondsPositionRequest request = new OpenSixtySecondsPositionRequest(
				positionType, 
				amount, 
				asset,
				seconds
			);
			Position position = tradingService.openSixtySecondsPosition(request);
			System.out.println("\t"+"Opened 60seconds position "+position); 
		} catch (TradingClientException e) {
			System.out.println("Exception on openSixtySecondsPosition, error:"+e.getTradingError());
		}
		
		/*9. request available regular EUR/USD options and open a regular position to the next available one*/
		List<RegularOption> availableOptions = null; 
		try {
			System.out.println("9. Requesting available regular EUR/USD options");
			String asset = "EUR/USD";
			RegularOptionsRequest regularOptionsRequest = new RegularOptionsRequest(asset); 
			availableOptions = tradingService.requestRegularOptions(regularOptionsRequest); 
			System.out.println("\tAvailable Regular Options for "+asset+":");
			for(RegularOption option: availableOptions){
				System.out.println("\t"+option);
			}
		}
		catch(TradingClientException e){
			System.out.println("Exception on openSixtySecondsPosition, error:"+e.getTradingError());
		}
		
		/*10. if there are available options, open a position to the next available one*/
		if(availableOptions!=null && !availableOptions.isEmpty()){
			RegularOption optionToOpen = availableOptions.iterator().next(); 
			String positionType = "call"; 
			double amount = 25;

			System.out.println("10. Sending regular position opening request");
			String expirationDate = optionToOpen.getEndDate(); 
			try{
				OpenRegularPositionRequest request = new OpenRegularPositionRequest(positionType, amount, expirationDate, optionToOpen.getAsset());
				Position position = tradingService.openRegularPosition(request);
				System.out.println("\tOpened regular position "+position);
			}
			catch(TradingClientException e){
				System.out.println("Exception on openRegularPosition, error:"+e.getTradingError());			
			}
		}
	}

}
